/*
 * Software License Agreement (BSD License)
 *
 * Copyright (c) 2010, Kyoto University and Honda Motor Co.,Ltd. All rights reserved.
 *
 * HARK was developed by researchers in Okuno Laboratory at the Kyoto University and 
 * Honda Research Institute Japan Co.,Ltd.
 *
 * Redistribution and use in source and binary forms, with or without modification, are 
 * permitted provided that the following conditions are met:
 *
 *  * Redistributions of source code must retain the above copyright notice, this list 
 *    of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright notice, this 
 *    list of conditions and the following disclaimer in the documentation and/or other 
 *    materials provided with the distribution.
 *  * Neither the name of Kyoto University and Honda Motor Co.,Ltd., Inc. nor the names 
 *    of its contributors may be used to endorse or promote products derived from this 
 *    software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY 
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
 * SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCI-
 * DENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSI-
 * NESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTR-
 * ACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "BufferedNode.h"
#include "operators.h"
#include "CvfdVideo.h"
#include "CvfdImage.h"

using namespace std;
using namespace FD;
   
class CvfdWebcam;
   
DECLARE_NODE(CvfdWebcam)
/*Node
 *
 * @name CvfdWebcam
 * @category FDOPENCV_ORG:Video
 * @description Load a video from a file
 *
 * @parameter_name INDEX
 * @parameter_type int
 * @parameter_value -1
 * @parameter_description Index of the camera to be used. If there is only one camera or it does not matter what camera to use -1 may be passed.
 * 
 * @output_name VIDEO
 * @output_description The loaded video data structure
 * @output_type CvfdVideo
 *
 END*/
   
   
class CvfdWebcam : public BufferedNode {
      
  //Output ID
  int m_videoID;
  //Parameters
  int m_index;      
      
public:
  CvfdWebcam(string nodeName, ParameterSet params)
    : BufferedNode(nodeName, params)
  {
    //add outputs
    m_videoID = addOutput("VIDEO");
    //Initialize parameters         
    m_index = dereference_cast<int>(parameters.get("INDEX"));
  }
      
      
  void calculate(int output_id, int count, Buffer &out)
  {
         
    //Handle
    RCPtr<CvfdVideo> videoPtr(new CvfdVideo(m_index));       
    out[count] = videoPtr;
  }
      
  NO_ORDER_NODE_SPEEDUP(CvfdWebcam)
};
